//
//  MCViewController.m
//  VideoFaceDetection
//
//  Created by Michael Conrads on 24.09.13.
//  Copyright (c) 2013 Michael Conrads. All rights reserved.
//

#import "MCViewController.h"
#import <CoreImage/CoreImage.h>
#import <ImageIO/ImageIO.h>
#import <AssertMacros.h>
#import <AssetsLibrary/AssetsLibrary.h>


@interface MCViewController ()

@property (nonatomic, strong) AVCaptureVideoDataOutput *videoDataOutput;
@property (nonatomic) dispatch_queue_t videoDataOutputQueue;
@property (nonatomic, strong) AVCaptureVideoPreviewLayer *previewLayer;

@property (nonatomic, strong) CIDetector *faceDetector;
@property (nonatomic, strong) UIView * boxView;

- (void)setupAVCapture;
- (void)teardownAVCapture;
- (void)drawFaces:(NSArray *)features forVideoBox:(CGRect)videoBox;

@end

@implementation MCViewController

- (void)setupAVCapture
{
	NSError *error = nil;
	AVCaptureSession *session = [[AVCaptureSession alloc] init];
	[session setSessionPreset:AVCaptureSessionPreset640x480];
	AVCaptureDevice *device;
    AVCaptureDevicePosition desiredPosition = AVCaptureDevicePositionFront;
	for (AVCaptureDevice *d in [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo]) {
		if ([d position] == desiredPosition) {
			device = d;
			break;
		}
	}
    
    AVCaptureDeviceInput *deviceInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
	if( !error ) {
        
        if ( [session canAddInput:deviceInput]){
            [session addInput:deviceInput];
        }
        
        self.videoDataOutput = [[AVCaptureVideoDataOutput alloc] init];
        NSDictionary *rgbOutputSettings = [NSDictionary dictionaryWithObject:
                                           [NSNumber numberWithInt:kCMPixelFormat_32BGRA] forKey:(id)kCVPixelBufferPixelFormatTypeKey];
        [self.videoDataOutput setVideoSettings:rgbOutputSettings];
        [self.videoDataOutput setAlwaysDiscardsLateVideoFrames:YES];
        self.videoDataOutputQueue = dispatch_queue_create("VideoDataOutputQueue", DISPATCH_QUEUE_SERIAL);
        [self.videoDataOutput setSampleBufferDelegate:self queue:self.videoDataOutputQueue];
        if ( [session canAddOutput:self.videoDataOutput] ){
            [session addOutput:self.videoDataOutput];
        }
       
        [[self.videoDataOutput connectionWithMediaType:AVMediaTypeVideo] setEnabled:YES];
		
        self.previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:session];
        self.previewLayer.backgroundColor = [[UIColor whiteColor] CGColor];
		self.previewLayer.videoGravity = AVLayerVideoGravityResizeAspect;
        
		CALayer *rootLayer = [self.previewView layer];
        [rootLayer setMasksToBounds:YES];
        [self.previewLayer setFrame:[rootLayer bounds]];
        [rootLayer addSublayer:self.previewLayer];
        [session startRunning];
        
    }
}



// find where the video box is positioned within the preview layer based on the video size
+ (CGRect)videoPreviewBoxForFrameSize:(CGSize)frameSize apertureSize:(CGSize)apertureSize
{
    CGFloat apertureRatio = apertureSize.height / apertureSize.width;
    CGFloat viewRatio = frameSize.width / frameSize.height;
    
    CGSize size = CGSizeZero;
	
	
	if (viewRatio > apertureRatio) {
		size.width = apertureSize.height * (frameSize.height / apertureSize.width);
		size.height = frameSize.height;
	} else {
		size.width = frameSize.width;
		size.height = apertureSize.width * (frameSize.width / apertureSize.height);
	}
	
	CGRect videoBox;
	videoBox.size = size;
	if (size.width < frameSize.width)
		videoBox.origin.x = (frameSize.width - size.width) / 2;
	else
		videoBox.origin.x = (size.width - frameSize.width) / 2;
	
	if ( size.height < frameSize.height )
		videoBox.origin.y = (frameSize.height - size.height) / 2;
	else
		videoBox.origin.y = (size.height - frameSize.height) / 2;
    
	return videoBox;
}


- (void)drawFaces:(NSArray *)features forVideoBox:(CGRect)clearAperture
{
	CGSize parentFrameSize = [self.previewView frame].size;
	
	CGRect previewBox = [MCViewController videoPreviewBoxForFrameSize:parentFrameSize apertureSize:clearAperture.size];
	
	for ( CIFaceFeature *ff in features ) {
		CGRect faceRect = [ff bounds];
		//the rect is rotate 90 deg, so we switch width/height and x/y
		CGFloat temp = faceRect.size.width;
		faceRect.size.width = faceRect.size.height;
		faceRect.size.height = temp;
		temp = faceRect.origin.x;
		faceRect.origin.x = faceRect.origin.y;
		faceRect.origin.y = temp;
	
		//adjust scaling
		CGFloat widthScaleBy = previewBox.size.width / clearAperture.size.height;
		CGFloat heightScaleBy = previewBox.size.height / clearAperture.size.width;
		faceRect.size.width *= widthScaleBy;
		faceRect.size.height *= heightScaleBy;
		faceRect.origin.x *= widthScaleBy;
		faceRect.origin.y *= heightScaleBy;
        
		faceRect = CGRectOffset(faceRect, previewBox.origin.x + previewBox.size.width - faceRect.size.width - (faceRect.origin.x * 2), previewBox.origin.y);
		
		self.boxView.frame = faceRect;
	}
	
	
}


#pragma mark Process video
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
	CVPixelBufferRef pixelBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
	CIImage *ciImage = [[CIImage alloc] initWithCVPixelBuffer:pixelBuffer];
	NSDictionary *imageOptions = [NSDictionary dictionaryWithObject:@6 forKey:CIDetectorImageOrientation];;
    
	//detect faces here
	NSArray *features = [self.faceDetector featuresInImage:ciImage options:imageOptions];
	
	
	CMFormatDescriptionRef fdesc = CMSampleBufferGetFormatDescription(sampleBuffer);
	CGRect cleanAperture = CMVideoFormatDescriptionGetCleanAperture(fdesc, false);
	
	dispatch_async(dispatch_get_main_queue(), ^(void) {
		[self drawFaces:features forVideoBox:cleanAperture];
	});
}




// clean up AVCapture
- (void)teardownAVCapture
{
	self.videoDataOutput = nil;
	[self.previewLayer removeFromSuperlayer];
	self.previewLayer = nil;
}


- (void)viewDidLoad
{
    [super viewDidLoad];
	
    self.boxView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 10, 10)];
	self.boxView.backgroundColor = [UIColor greenColor];
	self.boxView.alpha = 0.3;
	
	[self.view addSubview:self.boxView];
	
    [self setupAVCapture];
	
	//create detector
	NSDictionary *detectorOptions = [[NSDictionary alloc] initWithObjectsAndKeys:CIDetectorAccuracyLow, CIDetectorAccuracy, nil];
	self.faceDetector = [CIDetector detectorOfType:CIDetectorTypeFace context:nil options:detectorOptions];
}



@end
